#
# checkroot.sh	Check to root file system.
#
# Version:	@(#)checkroot.sh  2.85-10  16-Mar-2004  miquels@cistron.nl
#

. /etc/default/rcS

PATH=/lib/init:/bin:/sbin

#
# Set SULOGIN in /etc/default/rcS to yes if you want a sulogin to be spawned
# from this script *before anything else* with a timeout, like SCO does.
#
[ "$SULOGIN" = yes ] && sulogin -t 30 $CONSOLE

#
# Ensure that bdflush (update) is running before any major I/O is
# performed (the following fsck is a good example of such activity :).
#
if [ -x /sbin/update ]
then
	case `uname -r` in
		0.*|1.*|2.[0123].*)
			update
		;;
	esac
fi

#
# Read /etc/fstab.
#
exec 9>&0 </etc/fstab
rootmode=rw
rootopts=rw
rootcheck=yes
swap_on_md=no
devfs=
while read fs mnt type opts dump pass junk
do
	case "$fs" in
		""|\#*)
			continue;
			;;
		/dev/md*)
			# Swap on md device.
			[ "$type" = swap ] && swap_on_md=yes
			;;
		/dev)
			devfs="-t $type $fs $mnt"
			;;
		*)
			# Might be a swapfile.
			[ "$type" = swap ] && swap_on_md=yes
			;;
	esac
	[ "$mnt" != / ] && continue
	rootopts="$opts"
	[ "$pass" = 0 -o "$pass" = "" ] && rootcheck=no
	case "$opts" in
		ro|ro,*|*,ro|*,ro,*)
			rootmode=ro
			;;
	esac
done
exec 0>&9 9>&-

#
# Activate the swap device(s) in /etc/fstab. This needs to be done
# before fsck, since fsck can be quite memory-hungry.
#
doswap=no
case "`uname -r`" in
	2.[0123].*)
		if [ $swap_on_md = yes ] && grep -qs resync /proc/mdstat
		then
			[ "$VERBOSE" != no ] &&
			  echo "Not activating swap - RAID array resyncing"
		else
			doswap=yes
		fi
		;;
	*)
		doswap=yes
		;;
esac
if [ $doswap = yes ]
then
	[ "$VERBOSE" != no ] && echo "Activating swap."
	swapon -a 2> /dev/null
fi

#
# Check the root file system.
#
if [ -f /fastboot ] || [ $rootcheck = no ]
then
  [ $rootcheck = yes ] && echo "Fast boot, no file system check"
else
  #
  # Ensure that root is quiescent and read-only before fsck'ing.
  #
  mount -n -o remount,ro /
  if [ $? = 0 ]
  then
    if [ -f /forcefsck ]
    then
	force="-f"
    else
	force=""
    fi
    if [ "$FSCKFIX" = yes ]
    then
	fix="-y"
    else
	fix="-a"
    fi
    spinner="-C"
    case "$TERM" in
        dumb|network|unknown|"") spinner="" ;;
    esac
    [ `uname -m` = s390 ] && spinner="" # This should go away
    echo "Checking root file system..."
    fsck $spinner $force $fix /
    FSCKCODE=$?
    #
    # If there was a failure, drop into single-user mode.
    #
    # NOTE: "failure" is defined as exiting with a return code of
    # 4 or larger.  A return code of 1 indicates that file system
    # errors were corrected but that the boot may proceed. A return
    # code of 2 or 3 indicates that the system should immediately reboot.
    #
    if [ $FSCKCODE -gt 3 ]
    then
      # Surprise! Re-directing from a HERE document (as in
      # "cat << EOF") won't work, because the root is read-only.
      echo
      echo "fsck failed.  Please repair manually and reboot.  Please note"
      echo "that the root file system is currently mounted read-only.  To"
      echo "remount it read-write:"
      echo
      echo "   # mount -n -o remount,rw /"
      echo
      echo "CONTROL-D will exit from this shell and REBOOT the system."
      echo
      # Start a single user shell on the console
      /sbin/sulogin $CONSOLE
      reboot -f
    elif [ $FSCKCODE -gt 1 ]
    then
      echo "fsck corrected errors on the root partition, but requested that"
      echo "the system be rebooted."
      echo
      reboot -f
    fi
  else
    echo "*** ERROR!  Cannot fsck root fs because it is not mounted read-only!"
    echo
  fi
fi

#
#	Remount root to final mode (rw or ro). If it's rw, or if
#	/etc/mtab is a symlink (not to /proc/mounts) then we also
#	initialize the mtab file.
#	
mount -n -o remount,$rootmode /
if [ -L /etc/mtab ]
then
	# Get symlink destination file and directory.
	MTAB_PATH="`readlink -f /etc/mtab`"
else
	MTAB_PATH=/etc/mtab
fi
INIT_MTAB=
case "$MTAB_PATH" in
	/proc/*)
		;;
	/etc/mtab)
		# Only init on r/w root.
		if [ "$rootmode" = rw ]
		then
			:> /etc/mtab
			rm -f /etc/mtab~
			INIT_MTAB=Y
		fi
		;;
	"")
		echo "checkroot.sh: cannot initialize $MTAB_PATH" >&2
		;;
	*)
		:> $MTAB_PATH
		rm -f ${MTAB_PATH}~
		INIT_MTAB=Y
		;;
esac

if [ -n "$INIT_MTAB" ]
then
	mount -f -o remount /
	mount -f /proc
	[ -n "$devfs" ] && mount -f $devfs
	. /etc/init.d/mountvirtfs
fi

if [ -L /etc/nologin ]
then
	cd /etc
	rm -f `readlink nologin`
	cd /
elif [ "$rootmode" = rw ]
then
	rm -f /etc/nologin
fi

: exit 0

